/*
 *	app.h
 */
#ifndef __APP_H__
#define __APP_H__

#include "clip.h"
#include "usbcapt.h"	/* uPceCaps(v2001.12.18)Ήv */
#include "regsys.h"	/* uRegSysΉv */

/****************************************************************************
 *	`
 ****************************************************************************/

#define FPS	15			/* t[[g */
#define SEC(s)	((int)(FPS * (s)))	/* b->t[ϊ */

#define TRIAL_TERM_COUNT	50			/* I */
#define TRIAL_WARN_COUNT	(TRIAL_TERM_COUNT - 3)	/* x */
#define BATTLE_TERM_SEC		(3 * 60)		/* I */
#define BATTLE_WARN_SEC		(BATTLE_TERM_SEC - 10)	/* x */

/* Xe[g */
enum {
	STATE_REVERSE,	/* ʔ] */
	STATE_DEMO,	/* f */
	STATE_GAME,	/* Q[ */
	STATE_WIPE,	/* CvC/AEg */
	STATE_NEXT,	/* NEXTZ\ */
	STATE_FLDINF,	/* tB[h\ */
	STATE_RANKING,	/* LO\ */
	STATE_NAMEENTRY,/* l[Gg[ */
	N_STATES,	/* Xe[g̐ */
};

/* Z */
typedef struct CELL {
	char color;		/* F */
	char type;		/*  */
	char mode;		/* [h */
	char count;		/* Aj[VJE^ */
} CELL;
/* CELL.color */
#define CC_BLANK	0	/*  */
#define CC_WHITE	1	/*  */
#define CC_LIGHT	2	/*  */
#define CC_DARK		3	/*  */
#define CC_BLACK	4	/*  */
#define CC_CUBE		5	/* L[u */
#define CC_SUPER	6	/* X[p[ */
/* CELL.type */
#define CT_SINGLE	0	/* P */
#define CT_LEFT		1	/* () */
#define CT_RIGHT	2	/* (E) */
#define CT_UPPER	3	/* c() */
#define CT_UNDER	4	/* c() */
/* CELL.mode */
#define CM_STOP		0	/* ~ */
#define CM_FALL		1	/*  */
#define CM_BOUNCE	2	/* n */
#define CM_ERASE	3	/* Œ */
#define CM_DIATOM	4	/*  */
/* tB[h */
#define FIELD_COLS	8	/* tB[h̉Z */
#define FIELD_ROWS	13	/* tB[h̏cZ(ʓ12+ʊO1) */
#define FIELD_X0	 1	/* P1tB[hʒu */
#define FIELD_Y0	-7
#define FIELD_X1	70	/* P2tB[hʒu */
#define FIELD_Y1	-7
#define FLDINF_X0	 9	/* P1tB[hʒu */
#define FLDINF_Y0	 2
#define FLDINF_X1	 0	/* P2tB[hʒu(g܂) */
#define FLDINF_Y1	 2
#define ERASE_COUNT	3	/* Ȃ邩H */
typedef struct _FIELD {
	int side;		/* 0:P1/1:P2 */
//	int score;		/* XRA(gp) */
	CELL cells[FIELD_ROWS][FIELD_COLS];	/* tB[hZ */
	int cubes[FIELD_COLS];	/* ȇҋ@L[u */
	CELL quad[2][2];	/* Z */
	int x, y;		/* Z̍ʒu(sA) */
	int mode;		/* [h */
	FIXED count;		/* ėpJE^ */
	FIXED speed;		/* 䗎Xs[h */
	int combo;		/* ݂̘A */
	/* v */
	int ctrl_done;		/* 䊮(n+1) */
	int erase_num;		/* ŃZ(Jn̍v) */
	int combo_max;		/* őA */
} FIELD;
enum {
	FM_CTRL_INIT,		/* 䏉 */
	FM_CTRL_FALL,		/* 䗎 */
	FM_CTRL_BOUNCE,		/* 䒅n */
	FM_FALL,		/*  */
	FM_ERASE,		/* Œ */
	FM_CUBE_FALL,		/* L[uoE */
};

/* LO */
#define N_RANKS		8	/* ʂ܂ŋL^邩? */
typedef struct _RANK {
	int score;		/* XRA */
	char name[3 + 1];	/* O */
} RANK;
typedef struct _RANKING_DATA {
	RANK trial[N_RANKS];	/* gCA[h */
	RANK battle[N_RANKS];	/* og[h */
} RANKING_DATA;

/****************************************************************************
 *	ϐ
 ****************************************************************************/

extern char state[(N_STATES + 7) / 8];	/* Xe[g */
extern TASK* root_task;			/* [g^XN */
extern TASK* game_task;			/* Q[^XN */
extern int timer;			/* ^C} */
extern int pad;				/* pbh */
extern FIELD fields[2];			/* tB[h(0:P1/1:P2) */
extern CELL next[2][2];			/* NEXTZ */
extern unsigned short next_seed;	/* NEXT_V[h */
extern RANKING_DATA ranking_data;	/* LOf[^ */

/****************************************************************************
 *	֐
 ****************************************************************************/

/* main.c */
void initialize();
void finalize();
void period();

/* data.c */
extern const unsigned char PATTERN[];
extern const unsigned char BOMB1[];
extern const unsigned char BOMB2[];
extern const unsigned char BOMB3[];
extern const unsigned char BOMB4[];
extern const unsigned char CLICK1[];
extern const unsigned char CLICK2[];
extern const unsigned char CLICK3[];
extern const unsigned char WARN1[];
extern const unsigned char WIPEIN[];
extern const unsigned char WIPEOUT[];

/* misc.c */
int sprite_clear(int spr, int count);
void sprite_clear_all();
int sprite_print(int spr, int x, int y, const char* str);
int sprite_printf(int spr, int x, int y, const char* fmt, ...);
int sprite_block(int spr, int x, int y, int chr, int cols, int rows);
void bg_clear(int bg);
void bg_clear_all();
void bg_print(int bg, int x, int y, const char* str);
void bg_printf(int bg, int x, int y, const char* fmt, ...);
void bg_block(int bg, int x, int y, int chr, int cols, int rows);
void bg_pattern(int bg, int x, int y, int chr, int dot);
void sound_init();
void sound_exit();
void sound_start(int ch, const void* data);
void sound_stop(int ch);
const char* time_str(int t);

/* demo.c */
int demo();

/* ranking.c */
void ranking_init();
void ranking_exit();
void ranking(int type);
void nameentry(int type, int score);

/* game.c */
void game(int type);
void next_init();
void next_get(CELL quad[2][2]);

/* hamon.c */
#define N_HAMONS	16	/* g̍ő吔 */
TASK* t_hamon();
void hamon_new(int type, int x, int y);

/* wipe.c */
TASK* t_wipe(int type);	/* 0:CvAEg/1:CvC */

/* field.c */
int quad_draw(int spr, int x, int y, CELL quad[2][2]);
TASK* t_field(FIELD* field);
int field_fall_iteration(FIELD* field);
int field_count(FIELD* field, char mark[FIELD_ROWS], int x, int y);
int field_erase_start(FIELD* field);
int field_erase_iteration(FIELD* field);
void field_init(FIELD* field, int side);
int field_think(FIELD* field);
int field_proc(FIELD* field, int pad);

/* next.c */
TASK* t_next();

/* combo.c */
TASK* t_combo(int combo);

/****************************************************************************
 *	XvCg
 ****************************************************************************/

#define SPR_FLD0	0		/*   0-103(8*13):P1tB[h */
#define SPR_FLD1	104		/* 104-207(8*13):P2tB[h */
#define SPR_QUAD0	208		/* 208-211(2* 2):P1Rg[ */
#define SPR_QUAD1	212		/* 212-215(2* 2):P2Rg[ */
#define SPR_NEXTB	216		/* 216-224(3* 3):NEXT{[h */
#define SPR_NEXT	225		/* 225-223(2* 2):NEXTZ */
#define SPR_MSG		229		/* 229-244(1*16):bZ[W */
#define SPR_HAMON	245		/* 245-308(4*16):g */
#define N_SPRITES	309

#define CHR_TOFU1	0x001		/*  1x 1: () */
#define CHR_TOFU2	0x002		/*            () */
#define CHR_TOFU3	0x003		/*            () */
#define CHR_TOFU4	0x004		/*            () */
#define CHR_TOFU5	0x005		/*            (_) */

#define CHR_WIPE0	0x090		/*  1x 1: Cv */
#define CHR_WIPE1	0x091
#define CHR_WIPE2	0x092
#define CHR_WIPE3	0x093
#define CHR_WIPE4	0x094
#define CHR_WIPE5	0x095
#define CHR_WIPE6	0x096
#define CHR_WIPE7	0x097
#define CHR_TITLE	0x0E0		/* 12x 2: ^Cg */
#define CHR_FIELD	0x100		/* 16x11: tB[h */

#define CHR_SNG10	0x1B0		/*  1x 1: P */
#define CHR_SNG11	0x1C0
#define CHR_SNG12	0x1D0
#define CHR_SNG13	0x1E0
#define CHR_SNG14	0x1F0
#define CHR_SNG20	0x1B1		/*  1x 1: P */
#define CHR_SNG21	0x1C1
#define CHR_SNG22	0x1D1
#define CHR_SNG23	0x1E1
#define CHR_SNG24	0x1F1
#define CHR_SNG30	0x1B2		/*  1x 1: P */
#define CHR_SNG31	0x1C2
#define CHR_SNG32	0x1D2
#define CHR_SNG33	0x1E2
#define CHR_SNG34	0x1F2
#define CHR_SNG40	0x1B3		/*  1x 1: P */
#define CHR_SNG41	0x1C3
#define CHR_SNG42	0x1D3
#define CHR_SNG43	0x1E3
#define CHR_SNG44	0x1F3
#define CHR_SNG50	0x200		/*  1x 1: P */
#define CHR_SNG51	0x210
#define CHR_SNG52	0x220
#define CHR_SNG53	0x230
#define CHR_SNG54	0x240
#define CHR_SNG60	0x201		/*  1x 1: P */
#define CHR_SNG61	0x211
#define CHR_SNG62	0x221
#define CHR_SNG63	0x231
#define CHR_SNG64	0x241

#define CHR_HRZ10	0x1B4		/*  2x 1:  */
#define CHR_HRZ11	0x1C4
#define CHR_HRZ12	0x1D4
#define CHR_HRZ13	0x1E4
#define CHR_HRZ14	0x1F4
#define CHR_HRZ15	0x204
#define CHR_HRZ16	0x214
#define CHR_HRZ20	0x1B6		/*  2x 1:  */
#define CHR_HRZ21	0x1C6
#define CHR_HRZ22	0x1D6
#define CHR_HRZ23	0x1E6
#define CHR_HRZ24	0x1F6
#define CHR_HRZ25	0x206
#define CHR_HRZ26	0x216
#define CHR_HRZ30	0x1B8		/*  2x 1:  */
#define CHR_HRZ31	0x1C8
#define CHR_HRZ32	0x1D8
#define CHR_HRZ33	0x1E8
#define CHR_HRZ34	0x1F8
#define CHR_HRZ35	0x208
#define CHR_HRZ36	0x218
#define CHR_HRZ40	0x1BA		/*  2x 1:  */
#define CHR_HRZ41	0x1CA
#define CHR_HRZ42	0x1DA
#define CHR_HRZ43	0x1EA
#define CHR_HRZ44	0x1FA
#define CHR_HRZ45	0x20A
#define CHR_HRZ46	0x21A

#define CHR_VRT10	0x1BC		/*  1x 2: c */
#define CHR_VRT11	0x1DC
#define CHR_VRT12	0x1FC
#define CHR_VRT13	0x21C
#define CHR_VRT14	0x23C
#define CHR_VRT15	0x25C
#define CHR_VRT16	0x27C
#define CHR_VRT20	0x1BD		/*  1x 2: c */
#define CHR_VRT21	0x1DD
#define CHR_VRT22	0x1FD
#define CHR_VRT23	0x21D
#define CHR_VRT24	0x23D
#define CHR_VRT25	0x25D
#define CHR_VRT26	0x27D
#define CHR_VRT30	0x1BE		/*  1x 2: c */
#define CHR_VRT31	0x1DE
#define CHR_VRT32	0x1FE
#define CHR_VRT33	0x21E
#define CHR_VRT34	0x23E
#define CHR_VRT35	0x25E
#define CHR_VRT36	0x27E
#define CHR_VRT40	0x1BF		/*  1x 2: c */
#define CHR_VRT41	0x1DF
#define CHR_VRT42	0x1FF
#define CHR_VRT43	0x21F
#define CHR_VRT44	0x23F
#define CHR_VRT45	0x25F
#define CHR_VRT46	0x27F

#define CHR_HAMON10	0x224		/*  2x 2: ۂg */
#define CHR_HAMON11	0x226
#define CHR_HAMON12	0x228
#define CHR_HAMON13	0x22A
#define CHR_HAMON20	0x244		/*  2x 2: lpg */
#define CHR_HAMON21	0x246
#define CHR_HAMON22	0x248
#define CHR_HAMON23	0x24A
#define CHR_HAMON30	0x264		/*  2x 2: Ђ`g */
#define CHR_HAMON31	0x266
#define CHR_HAMON32	0x268
#define CHR_HAMON33	0x26A

#define CHR_NEXTB	0x250		/*  3x 3: NEXT{[h */

#endif /*__APP_H__*/
